@@ -13,7 +13,7 @@
 # limitations under the License.
 import os
 import warnings
-from typing import Any, Dict, Optional, Type
+from typing import Any, Dict, Optional, Type, Union
 
 import torch
 import torch.nn as nn
@@ -21,27 +21,18 @@ import torch.nn as nn
 from flash.audio.speech_recognition.backbone import SPEECH_RECOGNITION_BACKBONES
 from flash.audio.speech_recognition.collate import DataCollatorCTCWithPadding
 from flash.audio.speech_recognition.input import SpeechRecognitionDeserializer
-from flash.audio.speech_recognition.output_transform import (
-    SpeechRecognitionBackboneState,
-    SpeechRecognitionOutputTransform,
-)
+from flash.audio.speech_recognition.output_transform import SpeechRecognitionOutputTransform
 from flash.core.data.io.input import ServeInput
 from flash.core.data.io.input_transform import InputTransform
-from flash.core.data.states import CollateFn
+from flash.core.data.io.output import Output
 from flash.core.model import Task
 from flash.core.registry import FlashRegistry
 from flash.core.serve import Composition
 from flash.core.utilities.imports import _AUDIO_AVAILABLE, requires
-from flash.core.utilities.types import (
-    INPUT_TRANSFORM_TYPE,
-    LR_SCHEDULER_TYPE,
-    OPTIMIZER_TYPE,
-    OUTPUT_TRANSFORM_TYPE,
-    OUTPUT_TYPE,
-)
+from flash.core.utilities.types import INPUT_TRANSFORM_TYPE, LR_SCHEDULER_TYPE, OPTIMIZER_TYPE
 
 if _AUDIO_AVAILABLE:
-    from transformers import Wav2Vec2Processor
+    from transformers import AutoProcessor
 
 
 class SpeechRecognition(Task):
@@ -54,7 +45,6 @@ class SpeechRecognition(Task):
         learning_rate: Learning rate to use for training, defaults to ``1e-5``.
         optimizer: Optimizer to use for training.
         lr_scheduler: The LR scheduler to use during training.
-        output: The :class:`~flash.core.data.io.output.Output` to use when formatting prediction outputs.
     """
 
     backbones: FlashRegistry = SPEECH_RECOGNITION_BACKBONES
@@ -64,11 +54,10 @@ class SpeechRecognition(Task):
     def __init__(
         self,
         backbone: str = "facebook/wav2vec2-base-960h",
+        processor_backbone: str = None,
         optimizer: OPTIMIZER_TYPE = "Adam",
         lr_scheduler: LR_SCHEDULER_TYPE = None,
-        learning_rate: float = 1e-5,
-        output: OUTPUT_TYPE = None,
-        output_transform: OUTPUT_TRANSFORM_TYPE = SpeechRecognitionOutputTransform(),
+        learning_rate: Optional[float] = None,
     ):
         os.environ["TOKENIZERS_PARALLELISM"] = "TRUE"
         # disable HF thousand warnings
@@ -82,14 +71,16 @@ class SpeechRecognition(Task):
             optimizer=optimizer,
             lr_scheduler=lr_scheduler,
             learning_rate=learning_rate,
-            output=output,
-            output_transform=output_transform,
+            output_transform=SpeechRecognitionOutputTransform(backbone),
         )
 
         self.save_hyperparameters()
 
-        self.set_state(SpeechRecognitionBackboneState(backbone))
-        self.set_state(CollateFn(DataCollatorCTCWithPadding(Wav2Vec2Processor.from_pretrained(backbone))))
+        self.collate_fn = DataCollatorCTCWithPadding(
+            AutoProcessor.from_pretrained(backbone)
+            if processor_backbone is None
+            else AutoProcessor.from_pretrained(processor_backbone)
+        )
 
     def forward(self, batch: Dict[str, torch.Tensor]):
         return self.model(batch["input_values"])
@@ -111,5 +102,6 @@ class SpeechRecognition(Task):
         input_cls: Optional[Type[ServeInput]] = SpeechRecognitionDeserializer,
         transform: INPUT_TRANSFORM_TYPE = InputTransform,
         transform_kwargs: Optional[Dict] = None,
+        output: Optional[Union[str, Output]] = None,
     ) -> Composition:
-        return super().serve(host, port, sanity_check, input_cls, transform, transform_kwargs)
+        return super().serve(host, port, sanity_check, input_cls, transform, transform_kwargs, output)
