@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import os
-from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Type, TYPE_CHECKING, Union
+from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Type, Union
 
 import numpy as np
 import pytorch_lightning as pl
@@ -26,21 +26,19 @@ from torch.utils.data.sampler import Sampler
 import flash
 from flash.core.data.base_viz import BaseVisualization
 from flash.core.data.callback import BaseDataFetcher
-from flash.core.data.data_pipeline import DataPipeline, DataPipelineState
 from flash.core.data.io.input import DataKeys, Input, InputBase, IterableInput
-from flash.core.data.io.input_transform import InputTransform
+from flash.core.data.io.input_transform import (
+    _create_collate_input_transform_processors,
+    _InputTransformProcessorV2,
+    create_transform,
+    InputTransform,
+)
 from flash.core.data.io.output_transform import OutputTransform
 from flash.core.data.splits import SplitDataset
 from flash.core.data.utils import _STAGES_PREFIX
 from flash.core.registry import FlashRegistry
-from flash.core.utilities.imports import _FIFTYONE_AVAILABLE
 from flash.core.utilities.stages import RunningStage
 
-if _FIFTYONE_AVAILABLE and TYPE_CHECKING:
-    pass
-else:
-    SampleCollection = None
-
 
 class DatasetInput(Input):
     """The ``DatasetInput`` implements default behaviours for data sources which expect the input to
@@ -67,16 +65,13 @@ class DataModule(pl.LightningDataModule):
             :meth:`~flash.core.data.data_module.DataModule.configure_data_fetcher` will be used.
         val_split: An optional float which gives the relative amount of the training dataset to use for the validation
             dataset.
-        batch_size: The batch size to be used by the DataLoader. Defaults to 1.
+        batch_size: The batch size to be used by the DataLoader.
         num_workers: The number of workers to use for parallelized loading.
-            Defaults to None which equals the number of available CPU threads,
-            or 0 for Windows or Darwin platform.
         sampler: A sampler following the :class:`~torch.utils.data.sampler.Sampler` type.
             Will be passed to the DataLoader for the training dataset. Defaults to None.
     """
 
     input_transform_cls = InputTransform
-    output_transform_cls = OutputTransform
     input_transforms_registry: Optional[FlashRegistry] = None
 
     def __init__(
@@ -91,8 +86,7 @@ class DataModule(pl.LightningDataModule):
         num_workers: int = 0,
         sampler: Optional[Type[Sampler]] = None,
         pin_memory: bool = True,
-        persistent_workers: bool = True,
-        output_transform: Optional[OutputTransform] = None,
+        persistent_workers: bool = False,
     ) -> None:
 
         if not batch_size:
@@ -102,7 +96,6 @@ class DataModule(pl.LightningDataModule):
             batch_size = 16
 
         self._input_transform: Optional[OutputTransform] = None
-        self._output_transform: Optional[OutputTransform] = output_transform
         self._viz: Optional[BaseVisualization] = None
 
         self._train_input = train_input
@@ -110,6 +103,14 @@ class DataModule(pl.LightningDataModule):
         self._test_input = test_input
         self._predict_input = predict_input
 
+        if self._train_input and self._val_input and isinstance(val_split, float) and val_split > 0:
+            raise MisconfigurationException(
+                "A `val_dataset` was provided with `val_split`. Please, choose one or the other."
+            )
+
+        if self._train_input and (val_split is not None and not self._val_input):
+            self._train_input, self._val_input = self._split_train_val(self._train_input, val_split)
+
         self._data_fetcher: Optional[BaseDataFetcher] = data_fetcher or self.configure_data_fetcher()
 
         self._train_dataloader_collate_fn = self._resolve_dataloader_collate_fn(self._train_input)
@@ -117,18 +118,14 @@ class DataModule(pl.LightningDataModule):
         self._test_dataloader_collate_fn = self._resolve_dataloader_collate_fn(self._test_input)
         self._predict_dataloader_collate_fn = self._resolve_dataloader_collate_fn(self._predict_input)
 
-        self._train_on_after_batch_transfer_fn = self._resolve_on_after_batch_transfer_fn(self._train_input)
-        self._val_on_after_batch_transfer_fn = self._resolve_on_after_batch_transfer_fn(self._val_input)
-        self._test_on_after_batch_transfer_fn = self._resolve_on_after_batch_transfer_fn(self._test_input)
-        self._predict_on_after_batch_transfer_fn = self._resolve_on_after_batch_transfer_fn(self._predict_input)
-
-        if self._train_input and self._val_input and isinstance(val_split, float) and val_split > 0:
-            raise MisconfigurationException(
-                "A `val_dataset` was provided with `val_split`. Please, choose one or the other."
-            )
-
-        if self._train_input is not None and (val_split is not None and self._val_input is None):
-            self._train_input, self._val_input = self._split_train_val(self._train_input, val_split)
+        self._on_after_batch_transfer_fns = {
+            RunningStage.TRAINING: self._resolve_on_after_batch_transfer_fn(self._train_input),
+            RunningStage.VALIDATING: self._resolve_on_after_batch_transfer_fn(self._val_input),
+            RunningStage.SANITY_CHECKING: self._resolve_on_after_batch_transfer_fn(self._val_input),
+            RunningStage.TESTING: self._resolve_on_after_batch_transfer_fn(self._test_input),
+            RunningStage.PREDICTING: self._resolve_on_after_batch_transfer_fn(self._predict_input),
+        }
+        self._model_on_after_batch_transfer_fns = None
 
         if self._train_input:
             self.train_dataloader = self._train_dataloader
@@ -144,8 +141,6 @@ class DataModule(pl.LightningDataModule):
 
         self.batch_size = batch_size
 
-        if num_workers is None:
-            num_workers = 0
         self.num_workers = num_workers
         self.persistent_workers = persistent_workers and num_workers > 0
         self.pin_memory = pin_memory
@@ -174,11 +169,6 @@ class DataModule(pl.LightningDataModule):
         """This property returns the predict dataset."""
         return self._predict_input
 
-    def _resolve_transform(self, ds: Optional[Input]) -> Optional[InputTransform]:
-        if not isinstance(ds, Input):
-            return None
-        return ds.transform
-
     def _resolve_dataloader_collate_fn(self, ds: Optional[Input]) -> Optional[Callable]:
         if not ds:
             return None
@@ -194,7 +184,19 @@ class DataModule(pl.LightningDataModule):
 
     def _train_dataloader(self) -> DataLoader:
         train_ds: Input = self._train_input
+
         collate_fn = self._train_dataloader_collate_fn
+        if isinstance(getattr(self, "trainer", None), pl.Trainer):
+            input_transform = getattr(self.trainer.lightning_module, "input_transform", None)
+            if input_transform is not None:
+                input_transform = create_transform(input_transform, RunningStage.TRAINING)
+                collate_fn = _create_collate_input_transform_processors(input_transform, [self.data_fetcher])[0]
+
+        transform_processor = None
+        if isinstance(collate_fn, _InputTransformProcessorV2):
+            transform_processor = collate_fn
+            collate_fn = transform_processor.collate_fn
+
         shuffle: bool = False
         if isinstance(train_ds, IterableDataset):
             drop_last = False
@@ -208,9 +210,7 @@ class DataModule(pl.LightningDataModule):
             sampler = self.sampler(train_ds)
 
         if isinstance(getattr(self, "trainer", None), pl.Trainer):
-            if isinstance(self.trainer.lightning_module, flash.Task):
-                self.connect(self.trainer.lightning_module)
-            return self.trainer.lightning_module.process_train_dataset(
+            dataloader = self.trainer.lightning_module.process_train_dataset(
                 train_ds,
                 trainer=self.trainer,
                 batch_size=self.batch_size,
@@ -220,73 +220,126 @@ class DataModule(pl.LightningDataModule):
                 drop_last=drop_last,
                 collate_fn=collate_fn,
                 sampler=sampler,
+                persistent_workers=self.persistent_workers,
+            )
+        else:
+            dataloader = DataLoader(
+                train_ds,
+                batch_size=self.batch_size,
+                shuffle=shuffle,
+                sampler=sampler,
+                num_workers=self.num_workers,
+                pin_memory=self.pin_memory,
+                drop_last=drop_last,
+                collate_fn=collate_fn,
+                persistent_workers=self.persistent_workers,
             )
 
-        return DataLoader(
-            train_ds,
-            batch_size=self.batch_size,
-            shuffle=shuffle,
-            sampler=sampler,
-            num_workers=self.num_workers,
-            pin_memory=self.pin_memory,
-            drop_last=drop_last,
-            collate_fn=collate_fn,
-            persistent_workers=self.persistent_workers,
-        )
+        if transform_processor is not None:
+            transform_processor.collate_fn = dataloader.collate_fn
+            dataloader.collate_fn = transform_processor
+
+        self._model_on_after_batch_transfer_fns = None
+        return dataloader
 
     def _val_dataloader(self) -> DataLoader:
         val_ds: Input = self._val_input
+
         collate_fn = self._val_dataloader_collate_fn
+        if isinstance(getattr(self, "trainer", None), pl.Trainer):
+            input_transform = getattr(self.trainer.lightning_module, "input_transform", None)
+            if input_transform is not None:
+                input_transform = create_transform(input_transform, RunningStage.VALIDATING)
+                collate_fn = _create_collate_input_transform_processors(input_transform, [self.data_fetcher])[0]
+
+        transform_processor = None
+        if isinstance(collate_fn, _InputTransformProcessorV2):
+            transform_processor = collate_fn
+            collate_fn = transform_processor.collate_fn
 
         if isinstance(getattr(self, "trainer", None), pl.Trainer):
-            if isinstance(self.trainer.lightning_module, flash.Task):
-                self.connect(self.trainer.lightning_module)
-            return self.trainer.lightning_module.process_val_dataset(
+            dataloader = self.trainer.lightning_module.process_val_dataset(
                 val_ds,
                 trainer=self.trainer,
                 batch_size=self.batch_size,
                 num_workers=self.num_workers,
                 pin_memory=self.pin_memory,
                 collate_fn=collate_fn,
+                persistent_workers=self.persistent_workers,
+            )
+        else:
+            dataloader = DataLoader(
+                val_ds,
+                batch_size=self.batch_size,
+                num_workers=self.num_workers,
+                pin_memory=self.pin_memory,
+                collate_fn=collate_fn,
+                persistent_workers=self.persistent_workers,
             )
 
-        return DataLoader(
-            val_ds,
-            batch_size=self.batch_size,
-            num_workers=self.num_workers,
-            pin_memory=self.pin_memory,
-            collate_fn=collate_fn,
-            persistent_workers=self.persistent_workers,
-        )
+        if transform_processor is not None:
+            transform_processor.collate_fn = dataloader.collate_fn
+            dataloader.collate_fn = transform_processor
+
+        self._model_on_after_batch_transfer_fns = None
+        return dataloader
 
     def _test_dataloader(self) -> DataLoader:
         test_ds: Input = self._test_input
+
         collate_fn = self._test_dataloader_collate_fn
+        if isinstance(getattr(self, "trainer", None), pl.Trainer):
+            input_transform = getattr(self.trainer.lightning_module, "input_transform", None)
+            if input_transform is not None:
+                input_transform = create_transform(input_transform, RunningStage.TESTING)
+                collate_fn = _create_collate_input_transform_processors(input_transform, [self.data_fetcher])[0]
+
+        transform_processor = None
+        if isinstance(collate_fn, _InputTransformProcessorV2):
+            transform_processor = collate_fn
+            collate_fn = transform_processor.collate_fn
 
         if isinstance(getattr(self, "trainer", None), pl.Trainer):
-            if isinstance(self.trainer.lightning_module, flash.Task):
-                self.connect(self.trainer.lightning_module)
-            return self.trainer.lightning_module.process_test_dataset(
+            dataloader = self.trainer.lightning_module.process_test_dataset(
                 test_ds,
                 trainer=self.trainer,
                 batch_size=self.batch_size,
                 num_workers=self.num_workers,
                 pin_memory=self.pin_memory,
                 collate_fn=collate_fn,
+                persistent_workers=self.persistent_workers,
+            )
+        else:
+            dataloader = DataLoader(
+                test_ds,
+                batch_size=self.batch_size,
+                num_workers=self.num_workers,
+                pin_memory=self.pin_memory,
+                collate_fn=collate_fn,
+                persistent_workers=self.persistent_workers,
             )
 
-        return DataLoader(
-            test_ds,
-            batch_size=self.batch_size,
-            num_workers=self.num_workers,
-            pin_memory=self.pin_memory,
-            collate_fn=collate_fn,
-            persistent_workers=self.persistent_workers,
-        )
+        if transform_processor is not None:
+            transform_processor.collate_fn = dataloader.collate_fn
+            dataloader.collate_fn = transform_processor
+
+        self._model_on_after_batch_transfer_fns = None
+        return dataloader
 
     def _predict_dataloader(self) -> DataLoader:
         predict_ds: Input = self._predict_input
+
         collate_fn = self._predict_dataloader_collate_fn
+        if isinstance(getattr(self, "trainer", None), pl.Trainer):
+            input_transform = getattr(self.trainer.lightning_module, "input_transform", None)
+            if input_transform is not None:
+                input_transform = create_transform(input_transform, RunningStage.PREDICTING)
+                collate_fn = _create_collate_input_transform_processors(input_transform, [self.data_fetcher])[0]
+
+        transform_processor = None
+        if isinstance(collate_fn, _InputTransformProcessorV2):
+            transform_processor = collate_fn
+            collate_fn = transform_processor.collate_fn
 
         if isinstance(predict_ds, IterableDataset):
             batch_size = self.batch_size
@@ -294,60 +347,66 @@ class DataModule(pl.LightningDataModule):
             batch_size = min(self.batch_size, len(predict_ds) if len(predict_ds) > 0 else 1)
 
         if isinstance(getattr(self, "trainer", None), pl.Trainer):
-            if isinstance(self.trainer.lightning_module, flash.Task):
-                self.connect(self.trainer.lightning_module)
-            return self.trainer.lightning_module.process_predict_dataset(
+            dataloader = self.trainer.lightning_module.process_predict_dataset(
                 predict_ds,
                 batch_size=batch_size,
                 num_workers=self.num_workers,
                 pin_memory=self.pin_memory,
                 collate_fn=collate_fn,
+                persistent_workers=self.persistent_workers,
+            )
+        else:
+            dataloader = DataLoader(
+                predict_ds,
+                batch_size=batch_size,
+                num_workers=self.num_workers,
+                pin_memory=self.pin_memory,
+                collate_fn=collate_fn,
+                persistent_workers=self.persistent_workers,
             )
 
-        return DataLoader(
-            predict_ds,
-            batch_size=batch_size,
-            num_workers=self.num_workers,
-            pin_memory=self.pin_memory,
-            collate_fn=collate_fn,
-            persistent_workers=self.persistent_workers,
-        )
+        if transform_processor is not None:
+            transform_processor.collate_fn = dataloader.collate_fn
+            dataloader.collate_fn = transform_processor
 
-    def connect(self, task: "flash.Task"):
-        data_pipeline_state = DataPipelineState()
-        for properties in [
-            self._train_input,
-            self._val_input,
-            self._test_input,
-            self._predict_input,
-            getattr(self._train_input, "transform", None),
-            getattr(self._val_input, "transform", None),
-            getattr(self._test_input, "transform", None),
-            getattr(self._predict_input, "transform", None),
-            task._deserializer,
-            task._output_transform,
-            task._output,
-            task,
+        self._model_on_after_batch_transfer_fns = None
+        return dataloader
+
+    def _load_model_on_after_batch_transfer_fns(self) -> None:
+        self._model_on_after_batch_transfer_fns = {}
+
+        for stage in [
+            RunningStage.TRAINING,
+            RunningStage.VALIDATING,
+            RunningStage.SANITY_CHECKING,
+            RunningStage.TESTING,
+            RunningStage.PREDICTING,
         ]:
-            if properties is not None and hasattr(properties, "attach_data_pipeline_state"):
-                properties.attach_data_pipeline_state(data_pipeline_state)
+            transform = None
+            if isinstance(getattr(self, "trainer", None), pl.Trainer):
+                input_transform = getattr(self.trainer.lightning_module, "input_transform", None)
+                if input_transform is not None:
+                    input_transform = create_transform(
+                        input_transform, stage if stage != RunningStage.SANITY_CHECKING else RunningStage.VALIDATING
+                    )
+                    transform = _create_collate_input_transform_processors(input_transform, [self.data_fetcher])[1]
+            self._model_on_after_batch_transfer_fns[stage] = transform
 
     def on_after_batch_transfer(self, batch: Any, dataloader_idx: int) -> Any:
         if getattr(self, "trainer", None) is None:
             return batch
-        transform = None
-        if self.trainer.training:
-            transform = self._train_on_after_batch_transfer_fn
-        elif self.trainer.validating or self.trainer.sanity_checking:
-            transform = self._val_on_after_batch_transfer_fn
-        elif self.trainer.testing:
-            transform = self._test_on_after_batch_transfer_fn
-        elif self.trainer.predicting:
-            transform = self._predict_on_after_batch_transfer_fn
+
+        if self._model_on_after_batch_transfer_fns is None:
+            self._load_model_on_after_batch_transfer_fns()
+
+        stage = self.trainer.state.stage
+
+        transform = self._model_on_after_batch_transfer_fns[stage]
+        if transform is None:
+            transform = self._on_after_batch_transfer_fns[stage]
 
         if transform:
             batch = transform(batch)
-
         return batch
 
     @property
@@ -434,21 +493,27 @@ class DataModule(pl.LightningDataModule):
         stage_name: str = _STAGES_PREFIX[RunningStage.PREDICTING]
         self._show_batch(stage_name, hooks_names, reset=reset)
 
+    def _get_property(self, property_name: str) -> Optional[Any]:
+        train = getattr(self.train_dataset, property_name, None)
+        val = getattr(self.val_dataset, property_name, None)
+        test = getattr(self.test_dataset, property_name, None)
+        filtered = list(filter(lambda x: x is not None, [train, val, test]))
+        return filtered[0] if len(filtered) > 0 else None
+
     @property
     def num_classes(self) -> Optional[int]:
         """Property that returns the number of classes of the datamodule if a multiclass task."""
-        n_cls_train = getattr(self.train_dataset, "num_classes", None)
-        n_cls_val = getattr(self.val_dataset, "num_classes", None)
-        n_cls_test = getattr(self.test_dataset, "num_classes", None)
-        return n_cls_train or n_cls_val or n_cls_test
+        return self._get_property("num_classes")
+
+    @property
+    def labels(self) -> Optional[int]:
+        """Property that returns the labels if this ``DataModule`` contains classification data."""
+        return self._get_property("labels")
 
     @property
     def multi_label(self) -> Optional[bool]:
         """Property that returns ``True`` if this ``DataModule`` contains multi-label data."""
-        multi_label_train = getattr(self.train_dataset, "multi_label", None)
-        multi_label_val = getattr(self.val_dataset, "multi_label", None)
-        multi_label_test = getattr(self.test_dataset, "multi_label", None)
-        return multi_label_train or multi_label_val or multi_label_test
+        return self._get_property("multi_label")
 
     @property
     def inputs(self) -> Optional[Union[Input, List[InputBase]]]:
@@ -456,24 +521,6 @@ class DataModule(pl.LightningDataModule):
         inputs = [self.train_dataset, self.val_dataset, self.test_dataset, self.predict_dataset]
         return [input for input in inputs if input]
 
-    @property
-    def input_transform(self) -> InputTransform:
-        """Property that returns the input transform class used on input data."""
-        # Find a better way to resolve this.
-        return getattr(self.train_dataset, "transform", None) or self.input_transform_cls(RunningStage.TRAINING)
-
-    @property
-    def output_transform(self) -> OutputTransform:
-        """Property that returns the :class:`~flash.core.data.io.output_transform.OutputTransform` used to
-        output_transform the model outputs."""
-        return self._output_transform or self.output_transform_cls()
-
-    @property
-    def data_pipeline(self) -> DataPipeline:
-        """Property that returns the full data pipeline including the data source, input transform and
-        postprocessing."""
-        return DataPipeline(self.inputs, self.input_transform, self.output_transform)
-
     @staticmethod
     def _split_train_val(
         train_dataset: Dataset,
