@@ -4,7 +4,6 @@ import numpy as np
 from pytorch_lightning.utilities.exceptions import MisconfigurationException
 from torch.utils.data import Dataset
 
-import flash
 from flash.core.data.properties import Properties
 
 
@@ -29,8 +28,6 @@ class SplitDataset(Properties, Dataset):
         if isinstance(dataset, Properties):
             kwargs = dict(
                 running_stage=dataset._running_stage,
-                data_pipeline_state=dataset._data_pipeline_state,
-                state=dataset._state,
             )
         super().__init__(**kwargs)
 
@@ -50,11 +47,6 @@ class SplitDataset(Properties, Dataset):
         self.dataset = dataset
         self.indices = indices
 
-    def attach_data_pipeline_state(self, data_pipeline_state: "flash.core.data.data_pipeline.DataPipelineState"):
-        super().attach_data_pipeline_state(data_pipeline_state)
-        if isinstance(self.dataset, Properties):
-            self.dataset.attach_data_pipeline_state(data_pipeline_state)
-
     def __getattr__(self, key: str):
         if key != "dataset":
             return getattr(self.dataset, key)
