@@ -3,9 +3,13 @@ from typing import Any, Callable, Mapping
 
 import torch
 
-from flash.core.data.data_source import DefaultDataKeys
+from flash.core.data.batch import _ServeInputProcessor
+from flash.core.data.data_module import DataModule
+from flash.core.data.io.input import DataKeys
+from flash.core.data.io.output_transform import OutputTransform
 from flash.core.serve import expose, ModelComponent
 from flash.core.serve.types.base import BaseType
+from flash.core.trainer import Trainer
 from flash.core.utilities.stages import RunningStage
 
 
@@ -26,17 +30,17 @@ class FlashInputs(BaseType):
 class FlashOutputs(BaseType):
     def __init__(
         self,
-        serializer: Callable,
+        output: Callable,
     ):
-        self._serializer = serializer
+        self._output = output
 
     def serialize(self, outputs) -> Any:  # pragma: no cover
         results = []
         if isinstance(outputs, (list, torch.Tensor)):
             for output in outputs:
-                result = self._serializer(output)
+                result = self._output(output)
                 if isinstance(result, Mapping):
-                    result = result[DefaultDataKeys.PREDS]
+                    result = result[DataKeys.PREDS]
                 results.append(result)
         if len(results) == 1:
             return results[0]
@@ -46,36 +50,48 @@ class FlashOutputs(BaseType):
         return None
 
 
-def build_flash_serve_model_component(model):
+def build_flash_serve_model_component(model, serve_input, output):
+    # TODO: Resolve this hack
+    data_module = DataModule(predict_input=serve_input, batch_size=1)
 
-    data_pipeline = model.build_data_pipeline()
+    class MockTrainer(Trainer):
+        def __init__(self):
+            super().__init__()
+            self.state.stage = RunningStage.PREDICTING
+
+        @property
+        def lightning_module(self):
+            return model
+
+    data_module.trainer = MockTrainer()
+    dataloader = data_module.predict_dataloader()
+
+    collate_fn = dataloader.collate_fn
 
     class FlashServeModelComponent(ModelComponent):
         def __init__(self, model):
             self.model = model
             self.model.eval()
-            self.data_pipeline = model.build_data_pipeline()
-            self.worker_preprocessor = self.data_pipeline.worker_preprocessor(RunningStage.PREDICTING, is_serving=True)
-            self.device_preprocessor = self.data_pipeline.device_preprocessor(RunningStage.PREDICTING)
-            self.postprocessor = self.data_pipeline.postprocessor(RunningStage.PREDICTING, is_serving=True)
+            self.serve_input = serve_input
+            self.on_after_batch_transfer = data_module.on_after_batch_transfer
+            self.output_transform = getattr(model, "_output_transform", None) or OutputTransform()
             # todo (tchaton) Remove this hack
             self.extra_arguments = len(inspect.signature(self.model.transfer_batch_to_device).parameters) == 3
             self.device = self.model.device
 
         @expose(
-            inputs={"inputs": FlashInputs(data_pipeline.deserialize_processor())},
-            outputs={"outputs": FlashOutputs(data_pipeline.serialize_processor())},
+            inputs={"inputs": FlashInputs(_ServeInputProcessor(serve_input, collate_fn))},
+            outputs={"outputs": FlashOutputs(output)},
         )
         def predict(self, inputs):
             with torch.no_grad():
-                inputs = self.worker_preprocessor(inputs)
                 if self.extra_arguments:
                     inputs = self.model.transfer_batch_to_device(inputs, self.device, 0)
                 else:
                     inputs = self.model.transfer_batch_to_device(inputs, self.device)
-                inputs = self.device_preprocessor(inputs)
+                inputs = self.on_after_batch_transfer(inputs, 0)
                 preds = self.model.predict_step(inputs, 0)
-                preds = self.postprocessor(preds)
+                preds = self.output_transform(preds)
                 return preds
 
     return FlashServeModelComponent(model)
