@@ -11,6 +11,8 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import contextlib
+import functools
 import inspect
 import warnings
 from argparse import ArgumentParser, Namespace
@@ -21,13 +23,15 @@ import torch
 from pytorch_lightning import LightningDataModule, LightningModule
 from pytorch_lightning import Trainer as PlTrainer
 from pytorch_lightning.callbacks import BaseFinetuning
-from pytorch_lightning.loops.fit_loop import FitLoop
 from pytorch_lightning.utilities.argparse import add_argparse_args, get_init_arguments_and_types, parse_env_variables
 from pytorch_lightning.utilities.exceptions import MisconfigurationException
 from torch.utils.data import DataLoader
 
 import flash
+from flash.core.data.io.output import Output
+from flash.core.data.io.output_transform import OutputTransform
 from flash.core.model import Task
+from flash.core.registry import FlashRegistry
 
 
 def from_argparse_args(cls, args: Union[Namespace, ArgumentParser], **kwargs):
@@ -88,28 +92,6 @@ class Trainer(PlTrainer):
                 kwargs["precision"] = 32
         super().__init__(*args, **kwargs)
 
-    # TODO @(tchaton) remove `reset_train_val_dataloaders` from run_train function
-    def _run_train(self) -> None:
-        self._pre_training_routine()
-
-        if not self.is_global_zero and self.progress_bar_callback is not None:
-            self.progress_bar_callback.disable()
-
-        self._run_sanity_check(self.lightning_module)
-
-        # enable train mode
-        self.model.train()
-        torch.set_grad_enabled(True)
-
-        # reload data when needed
-        model = self.lightning_module
-
-        if isinstance(self.fit_loop, FitLoop):
-            self.reset_train_val_dataloaders(model)
-
-        self.fit_loop.trainer = self
-        self.fit_loop.run()
-
     def fit(
         self,
         model: LightningModule,
@@ -180,6 +162,57 @@ class Trainer(PlTrainer):
         self._resolve_callbacks(model, strategy, train_bn=train_bn)
         return super().fit(model, train_dataloader, val_dataloaders, datamodule)
 
+    @contextlib.contextmanager
+    def _wrap_predict_step(self, model, output_transform, output) -> None:
+        predict_step = model.predict_step
+
+        @functools.wraps(predict_step)
+        def wrapper(*args, **kwargs):
+            predictions = predict_step(*args, **kwargs)
+            if predictions is not None:
+                predictions = output_transform(predictions)
+                predictions = [output(prediction) for prediction in predictions]
+            return predictions
+
+        model.predict_step = wrapper
+        try:
+            yield
+        finally:
+            model.predict_step = predict_step
+
+    def predict(
+        self,
+        model: Optional[LightningModule] = None,
+        dataloaders: Optional[Union[DataLoader, LightningDataModule]] = None,
+        output: Union[Output, str] = None,
+        **kwargs,
+    ):
+        r"""
+        Run inference on your data.
+        This will call the model forward function to compute predictions. Useful to perform distributed
+        and batched predictions. Logging is disabled in the predict hooks.
+
+        Args:
+            model: The model to predict with.
+            dataloaders: A :class:`torch.utils.data.DataLoader` or a sequence of them,
+                or a :class:`~pytorch_lightning.core.datamodule.LightningDataModule` specifying prediction samples.
+            output: The :class:`~flash.core.data.io.output.Output` to use to transform predict outputs.
+            kwargs: Additional keyword arguments to pass to ``pytorch_lightning.Trainer.predict``.
+
+
+        Returns:
+            Returns a list of dictionaries, one for each provided dataloader containing their respective predictions.
+        """
+        model = model or self.lightning_module
+        output_transform = getattr(model, "_output_transform", None) or OutputTransform()
+        if output is None:
+            output = Output()
+        if isinstance(output, str) and isinstance(model, Task):
+            output = getattr(model, "outputs", FlashRegistry("outputs")).get(output).from_task(model)
+
+        with self._wrap_predict_step(model, output_transform, output):
+            return super().predict(model, dataloaders, **kwargs)
+
     def _resolve_callbacks(
         self,
         model: Task,
