@@ -15,16 +15,7 @@ import warnings
 from typing import Any, Dict, List, Optional
 
 from flash.core.adapter import AdapterTask
-from flash.core.data.data_source import DefaultDataKeys
-from flash.core.data.states import (
-    CollateFn,
-    PerBatchTransform,
-    PerBatchTransformOnDevice,
-    PerSampleTransformOnDevice,
-    PostTensorTransform,
-    PreTensorTransform,
-    ToTensorTransform,
-)
+from flash.core.data.io.input import DataKeys
 from flash.core.data.transforms import ApplyToKeys
 from flash.core.registry import FlashRegistry
 from flash.core.utilities.imports import _VISSL_AVAILABLE, requires
@@ -83,7 +74,7 @@ class ImageEmbedder(AdapterTask):
         pretrained: bool = False,
         optimizer: OPTIMIZER_TYPE = "Adam",
         lr_scheduler: LR_SCHEDULER_TYPE = None,
-        learning_rate: float = 1e-3,
+        learning_rate: Optional[float] = None,
         backbone_kwargs: Optional[Dict[str, Any]] = None,
         training_strategy_kwargs: Optional[Dict[str, Any]] = None,
         pretraining_transform_kwargs: Optional[Dict[str, Any]] = None,
@@ -119,19 +110,8 @@ class ImageEmbedder(AdapterTask):
             learning_rate=learning_rate,
         )
 
-        transform, collate_fn = self.transforms.get(pretraining_transform)(**pretraining_transform_kwargs)
-        to_tensor_transform = ApplyToKeys(
-            DefaultDataKeys.INPUT,
-            transform,
-        )
-
-        self.adapter.set_state(CollateFn(collate_fn))
-        self.adapter.set_state(ToTensorTransform(to_tensor_transform))
-        self.adapter.set_state(PostTensorTransform(None))
-        self.adapter.set_state(PreTensorTransform(None))
-        self.adapter.set_state(PerSampleTransformOnDevice(None))
-        self.adapter.set_state(PerBatchTransform(None))
-        self.adapter.set_state(PerBatchTransformOnDevice(None))
+        input_transform, self.collate_fn = self.transforms.get(pretraining_transform)(**pretraining_transform_kwargs)
+        self.input_transform = ApplyToKeys(DataKeys.INPUT, input_transform)
 
         warnings.warn(
             "Warning: VISSL ImageEmbedder overrides any user provided transforms"
