@@ -11,7 +11,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from typing import Callable, Optional
+from typing import Callable, Optional, Tuple
 
 from flash.core.utilities.flash_cli import FlashCLI
 from flash.core.utilities.imports import _ICEDATA_AVAILABLE, requires
@@ -25,11 +25,18 @@ __all__ = ["keypoint_detection"]
 
 @requires("image")
 def from_biwi(
+    train_folder: Optional[str] = None,
+    train_ann_file: Optional[str] = None,
+    val_folder: Optional[str] = None,
+    val_ann_file: Optional[str] = None,
+    test_folder: Optional[str] = None,
+    test_ann_file: Optional[str] = None,
+    predict_folder: Optional[str] = None,
     val_split: float = 0.1,
-    batch_size: int = 4,
-    num_workers: int = 0,
+    image_size: Tuple[int, int] = (128, 128),
     parser: Optional[Callable] = None,
-    **preprocess_kwargs,
+    batch_size: int = 1,
+    **data_module_kwargs,
 ) -> KeypointDetectionData:
     """Downloads and loads the BIWI data set from icedata."""
     data_dir = icedata.biwi.load_data()
@@ -37,13 +44,19 @@ def from_biwi(
     if parser is None:
         parser = icedata.biwi.parser
 
-    return KeypointDetectionData.from_folders(
-        train_folder=data_dir,
+    return KeypointDetectionData.from_icedata(
+        train_folder=train_folder or data_dir,
+        train_ann_file=train_ann_file,
+        val_folder=val_folder,
+        val_ann_file=val_ann_file,
+        test_folder=test_folder,
+        test_ann_file=test_ann_file,
+        predict_folder=predict_folder,
         val_split=val_split,
+        transform_kwargs=dict(image_size=image_size),
         batch_size=batch_size,
-        num_workers=num_workers,
         parser=parser,
-        **preprocess_kwargs,
+        **data_module_kwargs,
     )
 
 
