@@ -11,7 +11,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from typing import Any, Callable, Collection, Dict, List, Optional, Sequence, Type
+from typing import Any, Callable, Collection, Dict, List, Optional, Sequence, Type, Union
 
 import numpy as np
 import torch
@@ -19,10 +19,10 @@ import torch
 from flash.core.data.base_viz import BaseVisualization
 from flash.core.data.callback import BaseDataFetcher
 from flash.core.data.data_module import DataModule
-from flash.core.data.data_pipeline import DataPipelineState
-from flash.core.data.io.classification_input import ClassificationInput
+from flash.core.data.io.classification_input import ClassificationInputMixin
 from flash.core.data.io.input import DataKeys, Input
 from flash.core.data.io.input_transform import InputTransform
+from flash.core.data.utilities.classification import TargetFormatter
 from flash.core.data.utilities.samples import to_samples
 from flash.core.utilities.imports import _SKLEARN_AVAILABLE
 from flash.core.utilities.stages import RunningStage
@@ -34,17 +34,21 @@ else:
     Bunch = object
 
 
-class TemplateNumpyClassificationInput(ClassificationInput):
+class TemplateNumpyClassificationInput(Input, ClassificationInputMixin):
     """An example data source that records ``num_features`` on the dataset."""
 
     def load_data(
-        self, examples: Collection[np.ndarray], targets: Optional[Sequence[Any]] = None
+        self,
+        examples: Collection[np.ndarray],
+        targets: Optional[Sequence[Any]] = None,
+        target_formatter: Optional[TargetFormatter] = None,
     ) -> Sequence[Dict[str, Any]]:
         """Sets the ``num_features`` attribute and calls ``super().load_data``.
 
         Args:
             examples: The ``np.ndarray`` (num_examples x num_features).
             targets: Associated targets.
+            target_formatter: Optionally provide a ``TargetFormatter`` to control how targets are formatted.
 
         Returns:
             A sequence of samples / sample metadata.
@@ -52,7 +56,7 @@ class TemplateNumpyClassificationInput(ClassificationInput):
         if not self.predicting and isinstance(examples, np.ndarray):
             self.num_features = examples.shape[1]
         if targets is not None:
-            self.load_target_metadata(targets)
+            self.load_target_metadata(targets, target_formatter=target_formatter)
         return to_samples(examples, targets)
 
     def load_sample(self, sample: Dict[str, Any]) -> Any:
@@ -64,16 +68,17 @@ class TemplateNumpyClassificationInput(ClassificationInput):
 class TemplateSKLearnClassificationInput(TemplateNumpyClassificationInput):
     """An example data source that loads data from an sklearn data ``Bunch``."""
 
-    def load_data(self, data: Bunch) -> Sequence[Dict[str, Any]]:
+    def load_data(self, data: Bunch, target_formatter: Optional[TargetFormatter] = None) -> Sequence[Dict[str, Any]]:
         """Gets the ``data`` and ``target`` attributes from the ``Bunch`` and passes them to ``super().load_data``.
 
         Args:
             data: The scikit-learn data ``Bunch``.
+            target_formatter: Optionally provide a ``TargetFormatter`` to control how targets are formatted.
 
         Returns:
             A sequence of samples / sample metadata.
         """
-        return super().load_data(data.data, data.target)
+        return super().load_data(data.data, data.target, target_formatter=target_formatter)
 
     def predict_load_data(self, data: Bunch) -> Sequence[Dict[str, Any]]:
         """Avoid including targets when predicting.
@@ -95,11 +100,16 @@ class TemplateInputTransform(InputTransform):
         """Transform which creates a tensor from the given numpy ``ndarray`` and converts it to ``float``"""
         return torch.from_numpy(input).float()
 
+    @staticmethod
+    def target_to_tensor(target: Union[int, List[int]]):
+        """Transform which creates a tensor from the given target and casts it to ``long``"""
+        return torch.as_tensor(target).long()
+
     def input_per_sample_transform(self) -> Callable:
         return self.input_to_tensor
 
     def target_per_sample_transform(self) -> Callable:
-        return torch.as_tensor
+        return self.target_to_tensor
 
 
 class TemplateData(DataModule):
@@ -155,15 +165,31 @@ class TemplateData(DataModule):
         """
 
         ds_kw = dict(
-            data_pipeline_state=DataPipelineState(),
             transform_kwargs=transform_kwargs,
             input_transforms_registry=cls.input_transforms_registry,
         )
 
+        train_input = input_cls(RunningStage.TRAINING, train_data, train_targets, transform=train_transform, **ds_kw)
+        target_formatter = getattr(train_input, "target_formatter", None)
+
         return cls(
-            input_cls(RunningStage.TRAINING, train_data, train_targets, transform=train_transform, **ds_kw),
-            input_cls(RunningStage.VALIDATING, val_data, val_targets, transform=val_transform, **ds_kw),
-            input_cls(RunningStage.TESTING, test_data, test_targets, transform=test_transform, **ds_kw),
+            train_input,
+            input_cls(
+                RunningStage.VALIDATING,
+                val_data,
+                val_targets,
+                transform=val_transform,
+                target_formatter=target_formatter,
+                **ds_kw,
+            ),
+            input_cls(
+                RunningStage.TESTING,
+                test_data,
+                test_targets,
+                transform=test_transform,
+                target_formatter=target_formatter,
+                **ds_kw,
+            ),
             input_cls(RunningStage.PREDICTING, predict_data, transform=predict_transform, **ds_kw),
             **data_module_kwargs,
         )
@@ -203,17 +229,30 @@ class TemplateData(DataModule):
         Returns:
             The constructed data module.
         """
-
         ds_kw = dict(
-            data_pipeline_state=DataPipelineState(),
             transform_kwargs=transform_kwargs,
             input_transforms_registry=cls.input_transforms_registry,
         )
 
+        train_input = input_cls(RunningStage.TRAINING, train_bunch, transform=train_transform, **ds_kw)
+        target_formatter = getattr(train_input, "target_formatter", None)
+
         return cls(
-            input_cls(RunningStage.TRAINING, train_bunch, transform=train_transform, **ds_kw),
-            input_cls(RunningStage.VALIDATING, val_bunch, transform=val_transform, **ds_kw),
-            input_cls(RunningStage.TESTING, test_bunch, transform=test_transform, **ds_kw),
+            train_input,
+            input_cls(
+                RunningStage.VALIDATING,
+                val_bunch,
+                transform=val_transform,
+                target_formatter=target_formatter,
+                **ds_kw,
+            ),
+            input_cls(
+                RunningStage.TESTING,
+                test_bunch,
+                transform=test_transform,
+                target_formatter=target_formatter,
+                **ds_kw,
+            ),
             input_cls(RunningStage.PREDICTING, predict_bunch, transform=predict_transform, **ds_kw),
             **data_module_kwargs,
         )
