@@ -16,11 +16,10 @@ from typing import Any, Dict, List, Optional, Union
 
 import pandas as pd
 
-from flash.core.data.io.classification_input import ClassificationInput, ClassificationState
-from flash.core.data.io.input import DataKeys
-from flash.core.data.utilities.classification import TargetMode
+from flash.core.data.io.classification_input import ClassificationInputMixin
+from flash.core.data.io.input import DataKeys, Input
+from flash.core.data.utilities.classification import MultiBinaryTargetFormatter, TargetFormatter
 from flash.core.data.utilities.paths import PATH_TYPE
-from flash.core.integrations.transformers.states import TransformersBackboneState
 from flash.core.utilities.imports import _TEXT_AVAILABLE, requires
 
 if _TEXT_AVAILABLE:
@@ -29,7 +28,7 @@ else:
     Dataset = object
 
 
-class TextClassificationInput(ClassificationInput):
+class TextClassificationInput(Input, ClassificationInputMixin):
     @staticmethod
     def _resolve_target(target_keys: Union[str, List[str]], element: Dict[str, Any]) -> Dict[str, Any]:
         if not isinstance(target_keys, List):
@@ -44,20 +43,17 @@ class TextClassificationInput(ClassificationInput):
         hf_dataset: Dataset,
         input_key: str,
         target_keys: Optional[Union[str, List[str]]] = None,
-        max_length: int = 128,
+        target_formatter: Optional[TargetFormatter] = None,
     ) -> Dataset:
         """Loads data into HuggingFace datasets.Dataset."""
-        self.max_length = max_length
-
         if not self.predicting:
             hf_dataset = hf_dataset.map(partial(self._resolve_target, target_keys))
             targets = hf_dataset.to_dict()[DataKeys.TARGET]
-            self.load_target_metadata(targets)
+            self.load_target_metadata(targets, target_formatter=target_formatter)
 
             # If we had binary multi-class targets then we also know the labels (column names)
-            if self.target_mode is TargetMode.MULTI_BINARY and isinstance(target_keys, List):
-                classification_state = self.get_state(ClassificationState)
-                self.set_state(ClassificationState(target_keys, classification_state.num_classes))
+            if isinstance(self.target_formatter, MultiBinaryTargetFormatter) and isinstance(target_keys, List):
+                self.labels = target_keys
 
         # remove extra columns
         extra_columns = set(hf_dataset.column_names) - {input_key, DataKeys.TARGET}
@@ -69,13 +65,9 @@ class TextClassificationInput(ClassificationInput):
         return hf_dataset
 
     def load_sample(self, sample: Dict[str, Any]) -> Any:
-        tokenized_sample = self.get_state(TransformersBackboneState).tokenizer(
-            sample[DataKeys.INPUT], max_length=self.max_length, truncation=True, padding="max_length"
-        )
-        tokenized_sample = tokenized_sample.data
         if DataKeys.TARGET in sample:
-            tokenized_sample[DataKeys.TARGET] = self.format_target(sample[DataKeys.TARGET])
-        return tokenized_sample
+            sample[DataKeys.TARGET] = self.format_target(sample[DataKeys.TARGET])
+        return sample
 
 
 class TextClassificationCSVInput(TextClassificationInput):
@@ -85,10 +77,10 @@ class TextClassificationCSVInput(TextClassificationInput):
         csv_file: PATH_TYPE,
         input_key: str,
         target_keys: Optional[Union[str, List[str]]] = None,
-        max_length: int = 128,
+        target_formatter: Optional[TargetFormatter] = None,
     ) -> Dataset:
         dataset_dict = load_dataset("csv", data_files={"data": str(csv_file)})
-        return super().load_data(dataset_dict["data"], input_key, target_keys, max_length)
+        return super().load_data(dataset_dict["data"], input_key, target_keys, target_formatter=target_formatter)
 
 
 class TextClassificationJSONInput(TextClassificationInput):
@@ -99,10 +91,10 @@ class TextClassificationJSONInput(TextClassificationInput):
         field: str,
         input_key: str,
         target_keys: Optional[Union[str, List[str]]] = None,
-        max_length: int = 128,
+        target_formatter: Optional[TargetFormatter] = None,
     ) -> Dataset:
         dataset_dict = load_dataset("json", data_files={"data": str(json_file)}, field=field)
-        return super().load_data(dataset_dict["data"], input_key, target_keys, max_length)
+        return super().load_data(dataset_dict["data"], input_key, target_keys, target_formatter=target_formatter)
 
 
 class TextClassificationDataFrameInput(TextClassificationInput):
@@ -112,9 +104,11 @@ class TextClassificationDataFrameInput(TextClassificationInput):
         data_frame: pd.DataFrame,
         input_key: str,
         target_keys: Optional[Union[str, List[str]]] = None,
-        max_length: int = 128,
+        target_formatter: Optional[TargetFormatter] = None,
     ) -> Dataset:
-        return super().load_data(Dataset.from_pandas(data_frame), input_key, target_keys, max_length)
+        return super().load_data(
+            Dataset.from_pandas(data_frame), input_key, target_keys, target_formatter=target_formatter
+        )
 
 
 class TextClassificationParquetInput(TextClassificationInput):
@@ -124,9 +118,11 @@ class TextClassificationParquetInput(TextClassificationInput):
         parquet_file: PATH_TYPE,
         input_key: str,
         target_keys: Optional[Union[str, List[str]]] = None,
-        max_length: int = 128,
+        target_formatter: Optional[TargetFormatter] = None,
     ) -> Dataset:
-        return super().load_data(Dataset.from_parquet(str(parquet_file)), input_key, target_keys, max_length)
+        return super().load_data(
+            Dataset.from_parquet(str(parquet_file)), input_key, target_keys, target_formatter=target_formatter
+        )
 
 
 class TextClassificationListInput(TextClassificationInput):
@@ -135,10 +131,10 @@ class TextClassificationListInput(TextClassificationInput):
         self,
         inputs: List[str],
         targets: Optional[List[Any]] = None,
-        max_length: int = 128,
+        target_formatter: Optional[TargetFormatter] = None,
     ) -> Dataset:
         if targets is not None:
             hf_dataset = Dataset.from_dict({DataKeys.INPUT: inputs, DataKeys.TARGET: targets})
         else:
             hf_dataset = Dataset.from_dict({DataKeys.INPUT: inputs})
-        return super().load_data(hf_dataset, DataKeys.INPUT, DataKeys.TARGET, max_length)
+        return super().load_data(hf_dataset, DataKeys.INPUT, DataKeys.TARGET, target_formatter=target_formatter)
