@@ -13,17 +13,28 @@
 # limitations under the License.
 import os
 import warnings
-from typing import Any, Dict, List
+from typing import Any, Dict, List, Optional, Type, Union
 
 import torch
 from pytorch_lightning import Callback
 
-from flash.core.classification import ClassificationTask, Labels
-from flash.core.data.data_source import DefaultDataKeys
+from flash.core.classification import ClassificationTask
+from flash.core.data.io.input import DataKeys, ServeInput
+from flash.core.data.io.input_transform import InputTransform
+from flash.core.data.io.output import Output
 from flash.core.registry import FlashRegistry
-from flash.core.utilities.imports import _TRANSFORMERS_AVAILABLE
-from flash.core.utilities.types import LOSS_FN_TYPE, LR_SCHEDULER_TYPE, METRICS_TYPE, OPTIMIZER_TYPE, SERIALIZER_TYPE
+from flash.core.serve import Composition
+from flash.core.utilities.imports import _TRANSFORMERS_AVAILABLE, requires
+from flash.core.utilities.types import (
+    INPUT_TRANSFORM_TYPE,
+    LOSS_FN_TYPE,
+    LR_SCHEDULER_TYPE,
+    METRICS_TYPE,
+    OPTIMIZER_TYPE,
+)
 from flash.text.classification.backbones import TEXT_CLASSIFIER_BACKBONES
+from flash.text.classification.collate import TextClassificationCollate
+from flash.text.input import TextDeserializer
 from flash.text.ort_callback import ORTCallback
 
 if _TRANSFORMERS_AVAILABLE:
@@ -37,7 +48,8 @@ class TextClassifier(ClassificationTask):
 
     Args:
         num_classes: Number of classes to classify.
-        backbone: A model to use to compute text features can be any BERT model from HuggingFace/transformersimage .
+        backbone: A model to use to compute text features can be any BERT model from HuggingFace/transformersimage.
+        max_length: The maximum length to pad / truncate sequences to.
         optimizer: Optimizer to use for training.
         lr_scheduler: The LR scheduler to use during training.
         metrics: Metrics to compute for training and evaluation. Can either be an metric from the `torchmetrics`
@@ -46,7 +58,6 @@ class TextClassifier(ClassificationTask):
             `metric(preds,target)` and return a single scalar tensor. Defaults to :class:`torchmetrics.Accuracy`.
         learning_rate: Learning rate to use for training, defaults to `1e-3`
         multi_label: Whether the targets are multi-label or not.
-        serializer: The :class:`~flash.core.data.process.Serializer` to use when serializing prediction outputs.
         enable_ort: Enable Torch ONNX Runtime Optimization: https://onnxruntime.ai/docs/#onnx-runtime-for-training
     """
 
@@ -56,19 +67,23 @@ class TextClassifier(ClassificationTask):
 
     def __init__(
         self,
-        num_classes: int,
-        backbone: str = "prajjwal1/bert-tiny",
+        num_classes: Optional[int] = None,
+        labels: Optional[List[str]] = None,
+        backbone: str = "prajjwal1/bert-medium",
+        max_length: int = 128,
         loss_fn: LOSS_FN_TYPE = None,
         optimizer: OPTIMIZER_TYPE = "Adam",
         lr_scheduler: LR_SCHEDULER_TYPE = None,
         metrics: METRICS_TYPE = None,
-        learning_rate: float = 1e-2,
+        learning_rate: Optional[float] = None,
         multi_label: bool = False,
-        serializer: SERIALIZER_TYPE = None,
         enable_ort: bool = False,
     ):
         self.save_hyperparameters()
 
+        if labels is not None and num_classes is None:
+            num_classes = len(labels)
+
         os.environ["TOKENIZERS_PARALLELISM"] = "TRUE"
         # disable HF thousand warnings
         warnings.simplefilter("ignore")
@@ -84,31 +99,25 @@ class TextClassifier(ClassificationTask):
             metrics=metrics,
             learning_rate=learning_rate,
             multi_label=multi_label,
-            serializer=serializer or Labels(multi_label=multi_label),
+            labels=labels,
         )
         self.enable_ort = enable_ort
+        self.max_length = max_length
+        self.collate_fn = TextClassificationCollate(backbone=backbone, max_length=max_length)
         self.model = self.backbones.get(backbone)(num_labels=num_classes)
-        self.save_hyperparameters()
 
     @property
     def backbone(self):
         return self.model.base_model
 
     def forward(self, batch: Dict[str, torch.Tensor]):
-        return self.model(input_ids=batch.get("input_ids", None), attention_mask=batch.get("attention_mask", None))
-
-    def to_loss_format(self, x) -> torch.Tensor:
-        if isinstance(x, (SequenceClassifierOutput, Seq2SeqSequenceClassifierOutput)):
-            x = x.logits
-        return super().to_loss_format(x)
-
-    def to_metrics_format(self, x) -> torch.Tensor:
-        if isinstance(x, (SequenceClassifierOutput, Seq2SeqSequenceClassifierOutput)):
-            x = x.logits
-        return super().to_metrics_format(x)
+        result = self.model(input_ids=batch.get("input_ids", None), attention_mask=batch.get("attention_mask", None))
+        if isinstance(result, (SequenceClassifierOutput, Seq2SeqSequenceClassifierOutput)):
+            result = result.logits
+        return result
 
     def step(self, batch, batch_idx, metrics) -> dict:
-        target = batch.pop(DefaultDataKeys.TARGET)
+        target = batch.pop(DataKeys.TARGET)
         batch = (batch, target)
         return super().step(batch, batch_idx, metrics)
 
@@ -127,3 +136,16 @@ class TextClassifier(ClassificationTask):
         if self.enable_ort:
             callbacks.append(ORTCallback())
         return callbacks
+
+    @requires("serve")
+    def serve(
+        self,
+        host: str = "127.0.0.1",
+        port: int = 8000,
+        sanity_check: bool = True,
+        input_cls: Optional[Type[ServeInput]] = TextDeserializer,
+        transform: INPUT_TRANSFORM_TYPE = InputTransform,
+        transform_kwargs: Optional[Dict] = None,
+        output: Optional[Union[str, Output]] = None,
+    ) -> Composition:
+        return super().serve(host, port, sanity_check, input_cls, transform, transform_kwargs, output)
