@@ -12,27 +12,23 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 from types import FunctionType
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, Dict, Iterable, List, Optional, Union
 
 import torch
-from pytorch_lightning import LightningModule
-from pytorch_lightning.callbacks import Callback
-from pytorch_lightning.callbacks.finetuning import BaseFinetuning
 from pytorch_lightning.utilities.exceptions import MisconfigurationException
 from torch import nn
 from torch.nn import functional as F
-from torch.optim import Optimizer
 from torch.utils.data import DistributedSampler
 from torchmetrics import Accuracy
 
 import flash
-from flash.core.classification import ClassificationTask, Labels
-from flash.core.data.data_source import DefaultDataKeys
+from flash.core.classification import ClassificationTask
+from flash.core.data.io.input import DataKeys
 from flash.core.registry import FlashRegistry
 from flash.core.utilities.compatibility import accelerator_connector
 from flash.core.utilities.imports import _PYTORCHVIDEO_AVAILABLE
 from flash.core.utilities.providers import _PYTORCHVIDEO
-from flash.core.utilities.types import LOSS_FN_TYPE, LR_SCHEDULER_TYPE, METRICS_TYPE, OPTIMIZER_TYPE, SERIALIZER_TYPE
+from flash.core.utilities.types import LOSS_FN_TYPE, LR_SCHEDULER_TYPE, METRICS_TYPE, OPTIMIZER_TYPE
 
 _VIDEO_CLASSIFIER_BACKBONES = FlashRegistry("backbones")
 
@@ -46,32 +42,6 @@ if _PYTORCHVIDEO_AVAILABLE:
                 _VIDEO_CLASSIFIER_BACKBONES(fn=fn, providers=_PYTORCHVIDEO)
 
 
-class VideoClassifierFinetuning(BaseFinetuning):
-    def __init__(self, num_layers: int = 5, train_bn: bool = True, unfreeze_epoch: int = 1):
-        super().__init__()
-        self.num_layers = num_layers
-        self.train_bn = train_bn
-        self.unfreeze_epoch = unfreeze_epoch
-
-    def freeze_before_training(self, pl_module: LightningModule) -> None:
-        self.freeze(modules=list(pl_module.backbone.children())[: -self.num_layers], train_bn=self.train_bn)
-
-    def finetune_function(
-        self,
-        pl_module: LightningModule,
-        epoch: int,
-        optimizer: Optimizer,
-        opt_idx: int,
-    ) -> None:
-        if epoch != self.unfreeze_epoch:
-            return
-        self.unfreeze_and_add_param_group(
-            modules=list(pl_module.backbone.children())[-self.num_layers :],
-            optimizer=optimizer,
-            train_bn=self.train_bn,
-        )
-
-
 class VideoClassifier(ClassificationTask):
     """Task that classifies videos.
 
@@ -93,8 +63,6 @@ class VideoClassifier(ClassificationTask):
         head: either a `nn.Module` or a callable function that converts the features extrated from the backbone
             into class log probabilities (assuming default loss function). If `None`, will default to using
             a single linear layer.
-        serializer: A instance of :class:`~flash.core.data.process.Serializer` that determines how the output
-            should be serialized e.g. convert the model output into the desired output format when predicting.
     """
 
     backbones: FlashRegistry = _VIDEO_CLASSIFIER_BACKBONES
@@ -103,18 +71,23 @@ class VideoClassifier(ClassificationTask):
 
     def __init__(
         self,
-        num_classes: int,
+        num_classes: Optional[int] = None,
+        labels: Optional[List[str]] = None,
         backbone: Union[str, nn.Module] = "x3d_xs",
         backbone_kwargs: Optional[Dict] = None,
         pretrained: bool = True,
         loss_fn: LOSS_FN_TYPE = F.cross_entropy,
-        optimizer: OPTIMIZER_TYPE = "SGD",
+        optimizer: OPTIMIZER_TYPE = "Adam",
         lr_scheduler: LR_SCHEDULER_TYPE = None,
         metrics: METRICS_TYPE = Accuracy(),
-        learning_rate: float = 1e-3,
+        learning_rate: Optional[float] = None,
         head: Optional[Union[FunctionType, nn.Module]] = None,
-        serializer: SERIALIZER_TYPE = None,
     ):
+        self.save_hyperparameters()
+
+        if labels is not None and num_classes is None:
+            num_classes = len(labels)
+
         super().__init__(
             model=None,
             loss_fn=loss_fn,
@@ -122,11 +95,10 @@ class VideoClassifier(ClassificationTask):
             lr_scheduler=lr_scheduler,
             metrics=metrics,
             learning_rate=learning_rate,
-            serializer=serializer or Labels(),
+            num_classes=num_classes,
+            labels=labels,
         )
 
-        self.save_hyperparameters()
-
         if not backbone_kwargs:
             backbone_kwargs = {}
 
@@ -169,11 +141,12 @@ class VideoClassifier(ClassificationTask):
 
     def predict_step(self, batch: Any, batch_idx: int, dataloader_idx: int = 0) -> Any:
         predictions = self(batch["video"])
-        batch[DefaultDataKeys.PREDS] = predictions
+        batch[DataKeys.PREDS] = predictions
         return batch
 
-    def configure_finetune_callback(self) -> List[Callback]:
-        return [VideoClassifierFinetuning()]
+    def modules_to_freeze(self) -> Union[nn.Module, Iterable[Union[nn.Module, Iterable]]]:
+        """Return the module attributes of the model to be frozen."""
+        return list(self.backbone.children())
 
     @staticmethod
     def _ci_benchmark_fn(history: List[Dict[str, Any]]):
